#------------------------------------------------------------------------------
# human.R
#
# Author: Dustin Kapraun, US EPA, February 2018
#
# This code defines a function, "get_human_parms", that sets model parameters
# relevant to human anatomy, physiology, and physical/chemical properties (with
# respect to naphthalene). Here, the acslX M-file "human.m" that was supplied
# to EPA by JL Campbell on 2/13/2018 has been translated into R for use with
# the MCSim naphthalene models defined in "naph_pbtk_pde.model" and
# "naph_pbtk_2c.model". The user should specify which model versions to use by
# calling the "load_model" function defined in "RMCSim.R" or by initializing
# the R workspace by running (as "source") either "init_naph_pbtk_pde.R" or
# "init_naph_pbtk_2c.R".
#
# Reference:
# JL Campbell, ME Andersen, HJ Clewell, "A hybrid CFD-PBPK model for
# naphthalene in rat and human with IVIVE for nasal tissue metabolism and
# cross-species dosimetry," Inhalation Toxicology 26(6): 333-344, 2014.
#
# Specific changes and comments during translation from ACSL to MCSim are
# indicated by initials "DFK" for Dustin Kapraun.
#------------------------------------------------------------------------------

get_human_parms <- function() {
    # Set parameters to their default values as given in the ".model" file, and
    # then calculate values of those parameters that depend on other parameters.
    # Note that default parameter values in the ".model" file are for rats.
    parms = initParms()

    # Campbell-supplied code comment: "Taken from NTP Model except difu -
    # diffusivity for small molecules".
    DIFU = 0.00045
    parms["DIFU"] = DIFU

    # Campbell-supplied code comment: "Human PCs equal to rat".
    # Campbell-supplied code comment: "Hba estimated Willems = 571 (works well
    # with blood)".
    # NOTE: Values supplied for rat HTB and HPPB in Campbell-supplied source code
    # file "ratparam.m" (as well as default values in our "napth.model" file) are
    # slightly different from those given here. - DFK 2/26/2018
    HBA = 571.0
    parms["HBA"] = HBA
    HTB = 3.5
    parms["HTB"] = HTB
    HLB = 1.61
    parms["HLB"] = HLB
    HFB = 49.
    parms["HFB"] = HFB
    HRPB = 2.12
    parms["HRPB"] = HRPB
    HPPB = 3.5
    parms["HPPB"] = HPPB
    HLUB = 1.71
    parms["HLUB"] = HLUB

    # NOTE: These two parameters are defined in Campbell-supplied source code
    # file "human.m", but they are not used in "napth.csl". - DFK 2/26/2018
    # PAFA = 10. ;
    # FBLDV = 0.02 ;

    BWINIT = 70000.
    parms["BWinit"] = BWINIT
    QPUC = 1.2083
    parms["QPUc"] = QPUC
    MVC = 1.74278
    parms["MVc"] = MVC

    # Campbell-supplied code comment: "Tissue fractional volumes taken from
    # Brown et al. (1997)".
    FTABD = 0.022
    parms["FTABD"] = FTABD
    FTVBD = 0.045
    parms["FTVBD"] = FTVBD
    FTPP = 0.487
    parms["FTPP"] = FTPP
    FTRP = 0.1861
    parms["FTRP"] = FTRP
    FTREM = 0.09
    parms["FTREM"] = FTREM
    FTLI = 0.026
    parms["FTLI"] = FTLI
    FFAT = 0.136
    parms["FFAT"] = FFAT
    FBLI = 0.23
    parms["FBLI"] = FBLI
    FBRP = 0.44
    parms["FBRP"] = FBRP
    FBFA = 0.06
    parms["FBFA"] = FBFA
    FBURT = 0.0025
    parms["FBURT"] = FBURT
    FBCA = 0.01
    parms["FBCA"] = FBCA
    FBPP = 0.2575
    parms["FBPP"] = FBPP
    FADO = 0.15
    parms["FADO"] = FADO
    FAVR = 0.85
    parms["FAVR"] = FAVR
    FADR = 0.15
    parms["FADR"] = FADR

    # Campbell-supplied code comment: "Nasal Constants taken from Acetaldehyde
    # CFD-PBPK".
    WMUCUS = 0.001
    parms["WMUCUS"] = WMUCUS
    WTDR = 0.005
    parms["WTDR"] = WTDR
    WTDO = 0.004
    parms["WTDO"] = WTDO
    WTVR = 0.005
    parms["WTVR"] = WTVR
    WTPL = 0.0045
    parms["WTPL"] = WTPL
    WTCA = 0.0045
    parms["WTCA"] = WTCA
    WTPU = 0.001
    parms["WTPU"] = WTPU
    WXDR = 0.002
    parms["WXDR"] = WXDR
    WXDO = 0.002
    parms["WXDO"] = WXDO
    WXPL = 0.002
    parms["WXPL"] = WXPL
    WXVR = 0.002
    parms["WXVR"] = WXVR
    WXCA = 0.002
    parms["WXCA"] = WXCA

    SADR = 10.1
    parms["SADR"] = SADR
    SADO1 = 13.2
    parms["SADO1"] = SADO1
    SADO2 = 0.001
    parms["SADO2"] = SADO2
    SAVR1 = 42.1
    parms["SAVR1"] = SAVR1
    SAVR2 = 72.3
    parms["SAVR2"] = SAVR2
    SAPL = 192.3
    parms["SAPL"] = SAPL
    SACA = 2000
    parms["SACA"] = SACA
    SAPU = 540000
    parms["SAPU"] = SAPU

    VLCA = 95.7
    parms["VLCA"] = VLCA
    VLVR1 = 3.5
    parms["VLVR1"] = VLVR1
    VLVR2 = 5.16
    parms["VLVR2"] = VLVR2
    VLPL = 37.5
    parms["VLPL"] = VLPL
    VLDR = 0.74
    parms["VLDR"] = VLDR
    VLDO1 = 0.56
    parms["VLDO1"] = VLDO1
    VLDO2 = 0.001
    parms["VLDO2"] = VLDO2

    KGDR = 197
    parms["KGDR"] = KGDR
    KGDO1 = 1368
    parms["KGDO1"] = KGDO1
    KGDO2 = 0.01
    parms["KGDO2"] = KGDO2
    KGVR1 = 158
    parms["KGVR1"] = KGVR1
    KGVR2 = 194
    parms["KGVR2"] = KGVR2
    KGPL = 5711
    parms["KGPL"] = KGPL
    KGCA = 180
    parms["KGCA"] = KGCA


    # Campbell-supplied code comment: "Metabolic Parameters (nmol/mL/min and
    # nmol/ml)".
    VMLI = 100.8    # "sum of vmax in Cho et al. 2006".
    parms["VmaxLI"] = VMLI
    KMLI = 25.0     # "weighted average of 3 metabolites in Cho et al. 2006".
    parms["KmLI"] = KMLI
    VMLU = 0.27     # "Based on Buckpitt n Bahnson 1984 human lung metab".
    parms["VmaxLU"] = VMLU
    KMLU = 82.0     # "Based Monkey value from Van Winkle et al. 2012 or 2013
                    # (in submission)".
    parms["KmLU"] = KMLU

    # Campbell-supplied code comment: "Data from Van Winkle et al. 2012 or 2013
    # Monkey nasal epithelium".
    # "VMO 2 x's value and VMV 1/2 value total nose".
    VMO = 6.76
    parms["VmaxO"] = VMO
    VMV = 1.68
    parms["VmaxV"] = VMV
    KMRES = 11.5    # "rat value".
    parms["KmV"] = KMRES
    KMOLF = 70.0    # "rat value".
    parms["KmO"] = KMOLF

    # Recompute any parameters that may depend on the updated parameters.
    parms = initParms(parms)
    
    # Return vector of parameters.
    return(parms)
}