#------------------------------------------------------------------------------
# numerical_analysis.R
#
# Author: Dustin Kapraun, US EPA, November 2019
#
# This code performs a simulation of skin and "background" (intravenous)
# exposure using the model "naph_pbtk_pde". Most of the default human
# parameters are given in "human.R", but the default values of the parameters
# RIV, DSC, and HSCJP8 are taken to be 1.34 nmol/min, 9.09e-8 cm^2/min, and
# 0.204, respectively. These are the averages of the parameter values obtained
# when fitting the model to data from all of the Kim et al. (2006) subjects
# simultaneously. The default exposure time is taken to be 36 min, which is the
# mode of the exposure times recorded for the Kim et al. (2006) subjects. The
# simulation represents times up through 24 hours following the beginning of
# the skin exposure.
#
# The code computes the total mass of naphthalene in the system in two
# different ways: (1) by calculating the net mass of naphthalene that has moved
# into the organism via inhalation, exhalation, intravenous injection, and flux
# out of the exposure well (AIN); and (2) by calculating the total amount in
# organism, including the amount in the SC, plus the cumulative amount that has
# been metabolized (AINa).
#
# The code also computes the total mass of naphthalene in the SC for all times
# of interest during the simulation. This allows for comparison with other
# versions of this script that use more/less nodes to discretize the spatial
# variable that represents depth into the SC.
#
# References:
# D Kim, ME Andersen, LA Nylander-French (2006), "A dermatotoxicokinetic model
# of human exposures to jet fuel," Toxicological Sciences 93(1), 22-23.
#------------------------------------------------------------------------------

# Set working directory to the directory containing this file.
script_dir = dirname(sys.frame(1)$ofile)
setwd(script_dir)

# Load the model "naph_pbtk_pde".
source("init_naph_pbtk_pde.R")

# Save some colors from the viridis palette. This vector of color-identification
# strings was generated by calling the function "viridis" from the package
# "viridis" with the argument "5"; i.e., "viridis(5)".
vcol = c("#440154FF", "#3B528BFF", "#21908CFF", "#5DC863FF", "#FDE725FF")

# Define a function to run model simulations based on a parameter vector q.
run_sim <- function(q) {
    # Run a simulation for times up until 24 hours after the start of a skin
    # exposure. Return the model output for the period beginning at the start
    # of skin exposure and ending 24 hours thereafter.
    
    # Simulation times of interest.
    t_end = 24 * 60   # Minutes in one 24-hour period.
    times = seq(0, t_end, 1)
    
    # Remove elements of q that will be treated differently from other model
    # parameters and assign all other elements to the "parms" vector.
    AEXP = q["AEXP"]
    q["AEXP"] = 0.0
    exp_time = q["exp_time"]
    RIV = q["RIV"]
    CNJP8 = q["CNJP8"]
    parms = q[names(q) != "exp_time" & names(q) != "RIV"
              & names(q) != "CNJP8"]
    
    # Recalculate any parameters that depend on other parameters!
    parms = initParms(parms)
    
    # Get default initial conditions.
    Y0 = initStates(parms)
    
    # Modify default intravenous rate (nmol/min).
    Y0["RIV"] = RIV
    
    # Run a simulation assuming just the IV dosing to obtain steady state
    # conditions.
    TSTART = -1e5               # Start time (min).
    TSTOP = 0                   # Stop time (min).
    tvec = sort(c(TSTART, TSTOP, times[times < TSTOP]))
    Y0["RIV"] = RIV
    out0 = run_model(model_name, tvec, Y0, parms)
    
    # Reset initial conditions using final values of state variables from the
    # previous simulation.
    Y0 = out0[nrow(out0), names(Y0)]
    
    # Run a simulation for the exposure period.
    TSTART = 0.0                # Start time (min).
    TSTOP = exp_time            # Stop time (min).
    tvec = sort(c(TSTART, TSTOP, times[(times > TSTART) & (times < TSTOP)]))
    parms["AEXP"] = AEXP        # Area of exposed skin (cm^2).
    parms = initParms(parms)
    MW1 = parms[["MW1"]]
    VWELL = parms[["VWELL"]]
    Y0["AWELL"] = CNJP8 * VWELL / MW1 * 1e9   # Amount in well (nmol).
    out1 = run_model(model_name, tvec, Y0, parms)
    
    # Reset initial conditions using final values of state variables from the
    # previous simulation.
    Y0 = out1[nrow(out1), names(Y0)]
    
    # Run a simulation that encompasses all the observation times after the 
    # exposure period.
    TSTART = exp_time                   # Start time (min).
    TSTOP = max(c(exp_time + 1, times)) # Stop time (min).
    tvec = unique(sort(c(TSTART, TSTOP, times[times > TSTART])))
    parms["VWELL"] = 0.0        # Volume of skin exposure well (mL). Setting
                                # this to zero creates a zero flux condition at
                                # the outer surface of the stratum corneum.
    parms = initParms(parms)
    out2 = run_model(model_name, tvec, Y0, parms)
    
    # Package results in a single data frame.
    out = rbind(out0[1:nrow(out0)-1, ], out1[1:nrow(out1)-1, ], out2)
    out = out[out[ , "time"] %in% times, ]
    
    # Return simulation output.
    return(out)
}


# Initialize parameters with human default values.
q = get_human_parms()

# Modify relevant parameters.
q["AEXP"] = 20.0        # Area (cm^2) of exposed skin *during* skin exposure.
q["DSC"] = 9.09e-8      # SC diffusion parameter (cm^2/min).
q["HSCJP8"] = 0.204     # SC to JP-8 partition coefficient.
q = initParms(q)

# The following are also model parameters that determine skin exposure, but are
# not classified as "parameters" in the model
# specification file.
q["exp_time"] = 36.0    # Total duration (min) of skin exposure.
q["RIV"] = 1.34         # Rate of background (intravenous) exposure (nmol/min).
q["CNJP8"] = 0.003      # Concentration of naphthalene in JP-8 (g/mL).

# Run a simulation for the parameter values in q.
out = run_sim(q)

# Plot arterial blood concentration vs. time.
tout = out[, "time"] / 60.0                 # Simulation times (h).
CAB_ug_per_L = out[ , "CAB_UG"] * 1000.0    # Blood concentrations (ug/L).
plot(tout, CAB_ug_per_L, type="l", lwd=2, lty=1, col=vcol[1],
     xlab="Time (h)", ylab="Concentration in Blood (ug/L)")

# Plot venous blood concentration vs. time.
tout = out[, "time"] / 60.0                 # Simulation times (h).
CVB_ug_per_L = out[ , "CVB"] * q[["MW1"]]   # Blood concentrations (ug/L).
plot(tout, CVB_ug_per_L, type="l", lwd=2, lty=1, col=vcol[1],
     xlab="Time (h)", ylab="Concentration in Blood (ug/L)")

# Generate a figure that shows mass balance. Note that precision is limited by
# the finite difference approximation for flux (JSC00) that determines rate of
# flow into the SC from the exposure well (RADERM) and the trapezoid rule
# approximation for the total amount in the SC (ASC). The precision of these
# approximations is controlled by the step size (SCDX), so increasing the
# tolerances for the ODE integrator will not improve this precision.
AINa_vec = out[ , "AINa"]   # Amount in organism plus amount metabolized (nmol).
AINa_vec = AINa_vec * q[["MW1"]] / 1e6   # ... (mg)
AIN_vec = out[ , "AIN"]     # Net amount that has entered organism (nmol).
AIN_vec = AIN_vec * q[["MW1"]] / 1e6     # ... (mg)
AIN_avg = (AINa_vec + AIN_vec) / 2
AIN_diff = AINa_vec - AIN_vec
AIN_reld = 100 * AIN_diff / AIN_avg
plot(tout, AIN_diff, type="l", lwd=2, lty=1, col=vcol[1],
     xlab="Time (h)", ylab="Mass Balance (mg)")

# Generate a figure that shows relative error in the mass balance.
plot(tout, AIN_reld, type="l", lwd=2, lty=1, col=vcol[1],
     xlab="Time (h)", ylab="Relative Error in Mass Balance (%)")

# Print maximum relative error in naphthalene mass.
cat("Maximum percent difference between AINa and AIN:", max(AIN_reld), "%\n")
idx = which(AIN_reld == max(AIN_reld))
cat("At the time in which this difference occurs, the discprepancy\n",
    "is", AIN_diff[idx], "mg and the AIN estimate is", AIN_avg[idx], "mg.\n" )

# # Generate a figure that shows amount of naphthalene in the SC vs. time.
# ASC_ug = out[ , "ASC"] * q[["MW1"]] / 1000     # Mass in SC (ug).
# plot(tout, ASC_ug, type="l", lwd=2, lty=1, col=vcol[1],
#      xlab="Time (h)", ylab="Mass in SC (ug)")
# 
# # Save outputs for comparison with outputs obtained using more/less nodes to
# # discretize the spatial variable that represents depth into the SC.
# df_out = data.frame(tout=tout, CAB_ug_per_L=CAB_ug_per_L, ASC_ug=ASC_ug)
# saveRDS(df_out, file="./Output/df_out10.rds")