#------------------------------------------------------------------------------
# sensitivity_analysis.R
#
# Author: Dustin Kapraun, US EPA, September 2019
#
# This code performs a local sensitivity analysis for the model
# "naph_pbtk_pde". The sensitivities are calculated based on perturbations
# about a default human set of parameter values. Most of the default human
# parameters are given in "human.R", but the default values of the parameters
# RIV, DSC, and HSCJP8 are taken to be 1.34 nmol/min, 9.09e-8 cm^2/min, and
# 0.204, respectively. These are the averages of the parameter values obtained
# when fitting the model to data from all of the Kim et al. (2006) subjects
# simultaneously. The default exposure time is taken to be 36 min, which is the
# mode of the exposure times recorded for the Kim et al. (2006) subjects.
#
# References:
# D Kim, ME Andersen, LA Nylander-French (2006), "A dermatotoxicokinetic model
# of human exposures to jet fuel," Toxicological Sciences 93(1), 22-23.
#------------------------------------------------------------------------------

# Set working directory to the directory containing this file.
script_dir = dirname(sys.frame(1)$ofile)
setwd(script_dir)

# Load the model "naph_pbtk_pde".
source("init_naph_pbtk_pde.R")

# Save some colors from the viridis palette. This vector of color-identification
# strings was generated by calling the function "viridis" from the package
# "viridis" with the argument "5"; i.e., "viridis(5)".
vcol = c("#440154FF", "#3B528BFF", "#21908CFF", "#5DC863FF", "#FDE725FF")

# Define a function to run model simulations based on a parameter vector q.
run_sim <- function(q) {
    # Run a simulation for times up until 24 hours after the start of a skin
    # exposure. Return the model output for the period beginning at the start
    # of skin exposure and ending 24 hours thereafter.
    
    # Simulation times of interest.
    t_end = 24 * 60   # Minutes in one 24-hour period.
    times = seq(0, t_end, 1)
    
    # Remove elements of q that will be treated differently from other model
    # parameters and assign all other elements to the "parms" vector.
    AEXP = q["AEXP"]
    q["AEXP"] = 0.0
    exp_time = q["exp_time"]
    RIV = q["RIV"]
    CNJP8 = q["CNJP8"]
    parms = q[names(q) != "exp_time" & names(q) != "RIV"
              & names(q) != "CNJP8"]
    
    # Recalculate any parameters that depend on other parameters!
    parms = initParms(parms)
    
    # Get default initial conditions.
    Y0 = initStates(parms)
    
    # Modify default intravenous rate (nmol/min).
    Y0["RIV"] = RIV
    
    # Run a simulation assuming just the IV dosing to obtain steady state
    # conditions.
    TSTART = -1e5               # Start time (min).
    TSTOP = 0                   # Stop time (min).
    tvec = sort(c(TSTART, TSTOP, times[times < TSTOP]))
    Y0["RIV"] = RIV
    out0 = run_model(model_name, tvec, Y0, parms)
    
    # Reset initial conditions using final values of state variables from the
    # previous simulation.
    Y0 = out0[nrow(out0), names(Y0)]
    
    # Run a simulation for the exposure period.
    TSTART = 0.0                # Start time (min).
    TSTOP = exp_time            # Stop time (min).
    tvec = sort(c(TSTART, TSTOP, times[(times > TSTART) & (times < TSTOP)]))
    parms["AEXP"] = AEXP        # Area of exposed skin (cm^2).
    parms = initParms(parms)
    MW1 = parms[["MW1"]]
    VWELL = parms[["VWELL"]]
    Y0["AWELL"] = CNJP8 * VWELL / MW1 * 1e9   # Amount in well (nmol).
    out1 = run_model(model_name, tvec, Y0, parms)
    
    # Reset initial conditions using final values of state variables from the
    # previous simulation.
    Y0 = out1[nrow(out1), names(Y0)]
    
    # Run a simulation that encompasses all the observation times after the 
    # exposure period.
    TSTART = exp_time                   # Start time (min).
    TSTOP = max(c(exp_time + 1, times)) # Stop time (min).
    tvec = unique(sort(c(TSTART, TSTOP, times[times > TSTART])))
    parms["VWELL"] = 0.0        # Volume of skin exposure well (mL). Setting
                                # this to zero creates a zero flux condition at
                                # the outer surface of the stratum corneum.
    parms = initParms(parms)
    out2 = run_model(model_name, tvec, Y0, parms)
    
    # Package results in a single data frame.
    out = rbind(out0[1:nrow(out0)-1, ], out1[1:nrow(out1)-1, ], out2)
    out = out[out[ , "time"] %in% times, ]
    
    # Return simulation output.
    return(out)
}


comp_sens <- function(q, rel_step=1.0e-2) {
    # Ensure rel_step is positive.
    rel_step = abs(rel_step)

    # Run a simulation for the parameter values in q.
    out_q = run_sim(q)

    # Compute AUC for the blood concentration curve (ug/mL*min).
    AUC_q = out_q[[nrow(out), "AUC_CAB_UG"]] - out_q[[1, "AUC_CAB_UG"]]
    
    # Compute the average blood concentration (ug/L).
    avgC_q = 1000.0 * AUC_q /
        (out_q[[nrow(out), "time"]] - out_q[[1, "time"]])

    # Compute the difference between the average blood concentration and the
    # baseline blood concentration (ug/L).
    davgC_q = avgC_q - 1000.0 * out_q[[1, "CAB_UG"]]
    
    # Compute peak blood concentration (ug/L).
    peakC_q = 1000.0 * max(out_q[ , "CAB_UG"])
    
    # Compute the difference between the peak blood concentration and the
    # baseline blood concentration (ug/L).
    dpeakC_q = peakC_q - 1000.0 * out_q[[1, "CAB_UG"]]

    # Create a dataframe to store sensitivity indices.
    len_q = length(q)
    sens_df = data.frame(param_val=q,
                         Sn_avgC=double(len_q),
                         Sn_davgC=double(len_q),
                         Sn_peakC=double(len_q),
                         Sn_dpeakC=double(len_q))

    # For each parameter in q, compute a local sensitivity indices for both AUC
    # and peak blood concentration.
    for (qname in names(q)) {
        # Perturb this parameter by the "rel_step" amount.
        qx = q
        if (abs(q[qname]) >= rel_step / (1 + rel_step)) {
            qx[qname] = q[qname] * (1 + rel_step)
        } else if (q[qname] >= 0) {
            qx[qname] = rel_step
        } else {
            qx[qname] = -rel_step
        }

        # Run a simulation
        out_qx = run_sim(qx)

        # Compute AUC for the blood concentration curve (ug/mL*min).
        AUC_qx = out_qx[[nrow(out), "AUC_CAB_UG"]] - out_qx[[1, "AUC_CAB_UG"]]
        
        # Compute the average blood concentration (ug/L).
        avgC_qx = 1000.0 * AUC_qx /
            (out_qx[[nrow(out), "time"]] - out_qx[[1, "time"]])
        
        # Compute the difference between the average blood concentration and the
        # baseline blood concentration (ug/L).
        davgC_qx = avgC_qx - 1000.0 * out_qx[[1, "CAB_UG"]]
        
        # Compute peak blood concentration (ug/L).
        peakC_qx = 1000.0 * max(out_qx[ , "CAB_UG"])
        
        # Compute the difference between the peak blood concentration and the
        # baseline blood concentration (ug/L).
        dpeakC_qx = peakC_qx - 1000.0 * out_qx[[1, "CAB_UG"]]

        # Compute sensitivity indices for average blood concentration.
        sens_df[qname, "Sn_avgC"] = (avgC_qx - avgC_q) * q[qname] /
            ((qx[qname] - q[qname]) * avgC_q)
        sens_df[qname, "Sn_davgC"] = (davgC_qx - davgC_q) * q[qname] /
            ((qx[qname] - q[qname]) * davgC_q)

        # Compute sensitivity indices for peak concentration.
        sens_df[qname, "Sn_peakC"] = (peakC_qx - peakC_q) * q[qname] /
            ((qx[qname] - q[qname]) * peakC_q)
        sens_df[qname, "Sn_dpeakC"] = (dpeakC_qx - dpeakC_q) * q[qname] /
            ((qx[qname] - q[qname]) * dpeakC_q)
    }

    # Return the dataframe containing the sensitivity analysis results.
    return(sens_df)
}


# Initialize parameters with human default values.
q = get_human_parms()

# Modify relevant parameters.
q["AEXP"] = 20.0        # Area (cm^2) of exposed skin *during* skin exposure.
q["DSC"] = 9.09e-8      # SC diffusion parameter (cm^2/min).
q["HSCJP8"] = 0.204     # SC to JP-8 partition coefficient.
q = initParms(q)

# The following are also model parameters for which we will compute model
# sensitivities, but are not classified as "parameters" in the model
# specification file.
q["exp_time"] = 36.0    # Total duration (min) of skin exposure.
q["RIV"] = 1.34         # Rate of background (intravenous) exposure (nmol/min).
q["CNJP8"] = 0.003      # Concentration of naphthalene in JP-8 (g/mL).

# Run a simulation for the parameter values in q.
out = run_sim(q)

# Plot blood concentration vs. time.
tout = out[, "time"] / 60.0                 # Simulation times (h).
CAB_ug_per_L = out[ , "CAB_UG"] * 1000.0    # Blood concentrations (ug/L).
plot(tout, CAB_ug_per_L, type="l", lwd=2, lty=1, col=vcol[1],
     xlab="Time (h)", ylab="Concentration in Blood (ug/L)")

# Perform sensitivity analysis.
sens_df = comp_sens(q)

# Write sensitivity indices to a CSV file.
file_name = "./Output/sensitivity.csv"
write.csv(sens_df, file_name)

# Find all parameters (rows in the data frame) for which at least one of the
# normalized sensitivity indices is greater than 1.0.
sens_df_red = sens_df[rowSums(abs(sens_df[ , 2:5]) > 0.2) > 0, 2:5]

# Remove the parameter MW1 from consideration.
sens_df_red = sens_df_red[row.names(sens_df_red) != "MW1", ]

# Sort the rows based on values of Sn_avgC.
sens_df_red = sens_df_red[order(sens_df_red["Sn_avgC"]), ]

# Plot normalized sensitivity indices for average blood concentration.
Sn_avgC_vec = as.vector(sens_df[["Sn_avgC"]])
names(Sn_avgC_vec) = row.names(sens_df)
Sn_avgC_vec = Sn_avgC_vec[order(-abs(Sn_avgC_vec))]
plot(Sn_avgC_vec)

# Sn_avgC_vec = as.vector(sens_df_red[["Sn_avgC"]])
# names(Sn_avgC_vec) = row.names(sens_df_red)
# file_name = "./Output/Sn_avgC.tif"
# tiff(file_name, units="in", width=5, height=6, res=300)
# par(mar=c(5, 6, 4, 1) + 0.1)
# barplot(height=Sn_avgC_vec, horiz=TRUE, col=vcol[1], border=NA,
#         cex.names=0.9, las=2, xlim=c(-3.5, 2.5),
#         xlab="Sensitivity Index for Average Blood Concentration")
# dev.off()

# Plot normalized sensitivity indices for difference between average blood
# concentration and baseline blood concentration.
Sn_davgC_vec = as.vector(sens_df[["Sn_davgC"]])
names(Sn_davgC_vec) = row.names(sens_df)
Sn_davgC_vec = Sn_davgC_vec[order(-abs(Sn_davgC_vec))]
plot(Sn_davgC_vec)

# Sn_davgC_vec = as.vector(sens_df_red[["Sn_davgC"]])
# names(Sn_davgC_vec) = row.names(sens_df_red)
# file_name = "./Output/Sn_davgC.tif"
# tiff(file_name, units="in", width=5, height=6, res=300)
# par(mar=c(5, 6, 4, 1) + 0.1)
# barplot(height=Sn_davgC_vec, horiz=TRUE, col=vcol[1], border=NA,
#         cex.names=0.9, las=2, xlim=c(-3.5, 2.5),
#         xlab="Sensitivity Index for Average Blood Concentration")
# dev.off()

# Plot normalized sensitivity indices for peak blood concentration.
Sn_peakC_vec = as.vector(sens_df[["Sn_peakC"]])
names(Sn_peakC_vec) = row.names(sens_df)
Sn_peakC_vec = Sn_peakC_vec[order(-abs(Sn_peakC_vec))]
plot(Sn_peakC_vec)

# Sn_peakC_vec = as.vector(sens_df_red[["Sn_peakC"]])
# names(Sn_peakC_vec) = row.names(sens_df_red)
# file_name = "./Output/Sn_peakC.tif"
# tiff(file_name, units="in", width=5, height=6, res=300)
# par(mar=c(5, 6, 4, 1) + 0.1)
# barplot(height=Sn_peakC_vec, horiz=TRUE, col=vcol[1], border=NA,
#         cex.names=0.9, las=2, xlim=c(-3.5, 2.5),
#         xlab="Sensitivity Index for Peak Blood Concentration")
# dev.off()

# Plot normalized sensitivity indices for difference between peak blood
# concentration and baseline blood concentration.
Sn_dpeakC_vec = as.vector(sens_df[["Sn_dpeakC"]])
names(Sn_dpeakC_vec) = row.names(sens_df)
Sn_dpeakC_vec = Sn_dpeakC_vec[order(-abs(Sn_dpeakC_vec))]
plot(Sn_dpeakC_vec)

# Sn_dpeakC_vec = as.vector(sens_df_red[["Sn_dpeakC"]])
# names(Sn_dpeakC_vec) = row.names(sens_df_red)
# file_name = "./Output/Sn_dpeakC.tif"
# tiff(file_name, units="in", width=5, height=6, res=300)
# par(mar=c(5, 6, 4, 1) + 0.1)
# barplot(height=Sn_dpeakC_vec, horiz=TRUE, col=vcol[1], border=NA,
#         cex.names=0.9, las=2, xlim=c(-3.5, 2.5),
#         xlab="Sensitivity Index for Peak Blood Concentration")
# dev.off()

# Plot normalized sensitivity indices for average blood concentration and its
# difference from baseline blood concentration side by side.
file_name = "./Output/Sn_avgC2.tif"
tiff(file_name, units="in", width=5, height=6, res=300)
par(mar=c(5, 6, 4, 1) + 0.1)
barplot(t(as.matrix(sens_df_red[c(2, 1)])), horiz=TRUE, col=vcol[c(4, 1)],
        border=NA, beside=TRUE, las=2, xlim=c(-1.0, 1.0),
        xlab="Normalized Local Sensitivity Index")
legend("bottomright", legend=c("Dose Metric 1", "Dose Metric 2"),
       fill=vcol[c(1, 4)], border=NA, box.lty=1, cex=0.9)
dev.off()

# Plot normalized sensitivity indices for peak blood concentration and its
# difference from baseline blood concentration side by side.
file_name = "./Output/Sn_peakC2.tif"
tiff(file_name, units="in", width=5, height=6, res=300)
par(mar=c(5, 6, 4, 1) + 0.1)
barplot(t(as.matrix(sens_df_red[c(4, 3)])), horiz=TRUE, col=vcol[c(4, 1)],
        border=NA, beside=TRUE, las=2, xlim=c(-1.0, 1.0),
        xlab="Normalized Local Sensitivity Index")
legend("bottomright", legend=c("Dose Metric 3", "Dose Metric 4"),
       fill=vcol[c(1, 4)], border=NA, box.lty=1, cex=0.9)
dev.off()


# Plot normalized sensitivity indices for average blood concentration, its
# difference from baseline blood concentration, peak blood concentration and
# its difference from baseline blood concentration, all side-by-side.
file_name = "./Output/Sn_all.tif"
tiff(file_name, units="in", width=5, height=6, res=300)
par(mar=c(5, 6, 4, 1) + 0.1)
barplot(t(as.matrix(sens_df_red[c(4, 3, 2, 1)])), horiz=TRUE,
        col=vcol[c(4, 3, 2, 1)],
        border=NA, beside=TRUE, las=2, xlim=c(-1.0, 1.0),
        xlab="Normalized Local Sensitivity Index")
legend("bottomright", legend=c("Dose Metric 1", "Dose Metric 2",
                               "Dose Metric 3", "Dose Metric 4"),
       fill=vcol[c(1, 2, 3, 4)], border=NA, box.lty=1, cex=0.9)
dev.off()