#------------------------------------------------------------------------------
# skin_concentration_profile.R
#
# Author: Dustin Kapraun, US EPA, September 2019
#
# This code performs a simulation of skin exposure using the model
# "naph_pbtk_pde" and uses the results to produce figures showing skin
# skin concentration profiles (i.e., concentrations at various depths in the
# stratum corneum) for various points in time during the simulation. The
# simulations were conducted based on a default human set of parameter values.
# Most of the default human parameters are given in "human.R", but the default
# values of the parameters RIV, DSC, and HSCJP8 are taken to be 1.34 nmol/min,
# 9.09e-8 cm^2/min, and 0.204, respectively. These are the averages of the
# parameter values obtained when fitting the model to data from all of the Kim
# et al. (2006) subjects simultaneously. The default exposure time is taken to
# be 36 min, which is the mode of the exposure times recorded for the Kim et al.
# (2006) subjects.
#
# References:
# D Kim, ME Andersen, LA Nylander-French (2006), "A dermatotoxicokinetic model
# of human exposures to jet fuel," Toxicological Sciences 93(1), 22-23.
#------------------------------------------------------------------------------

# Set working directory to the directory containing this file.
script_dir = dirname(sys.frame(1)$ofile)
setwd(script_dir)

# Load the model "naph_pbtk_pde".
source("init_naph_pbtk_pde.R")

# Save some colors from the viridis palette. This vector of color-identification
# strings was generated by calling the function "viridis" from the package
# "viridis" with the argument "5"; i.e., "viridis(5)".
vcol = c("#440154FF", "#3B528BFF", "#21908CFF", "#5DC863FF", "#FDE725FF")

# Define a function to run model simulations based on a parameter vector q.
run_sim <- function(q) {
    # Run a simulation for times up until 24 hours after the start of a skin
    # exposure. Return the model output for the period beginning at the start
    # of skin exposure and ending 24 hours thereafter.
    
    # Simulation times of interest.
    t_end = 24 * 60   # Minutes in one 24-hour period.
    times = seq(0, t_end, 0.5)
    
    # Remove elements of q that will be treated differently from other model
    # parameters and assign all other elements to the "parms" vector.
    AEXP = q["AEXP"]
    q["AEXP"] = 0.0
    exp_time = q["exp_time"]
    RIV = q["RIV"]
    CNJP8 = q["CNJP8"]
    parms = q[names(q) != "exp_time" & names(q) != "RIV"
              & names(q) != "CNJP8"]
    
    # Recalculate any parameters that depend on other parameters!
    parms = initParms(parms)
    
    # Get default initial conditions.
    Y0 = initStates(parms)
    
    # Modify default intravenous rate (nmol/min).
    Y0["RIV"] = RIV
    
    # Run a simulation assuming just the IV dosing to obtain steady state
    # conditions.
    TSTART = -1e5               # Start time (min).
    TSTOP = 0                   # Stop time (min).
    tvec = sort(c(TSTART, TSTOP, times[times < TSTOP]))
    Y0["RIV"] = RIV
    out0 = run_model(model_name, tvec, Y0, parms)
    
    # Reset initial conditions using final values of state variables from the
    # previous simulation.
    Y0 = out0[nrow(out0), names(Y0)]
    
    # Run a simulation for the exposure period.
    TSTART = 0.0                # Start time (min).
    TSTOP = exp_time            # Stop time (min).
    tvec = sort(c(TSTART, TSTOP, times[(times > TSTART) & (times < TSTOP)]))
    parms["AEXP"] = AEXP        # Area of exposed skin (cm^2).
    parms = initParms(parms)
    MW1 = parms[["MW1"]]
    VWELL = parms[["VWELL"]]
    Y0["AWELL"] = CNJP8 * VWELL / MW1 * 1e9   # Amount in well (nmol).
    out1 = run_model(model_name, tvec, Y0, parms)
    
    # Reset initial conditions using final values of state variables from the
    # previous simulation.
    Y0 = out1[nrow(out1), names(Y0)]
    
    # Run a simulation that encompasses all the observation times after the 
    # exposure period.
    TSTART = exp_time                   # Start time (min).
    TSTOP = max(c(exp_time + 1, times)) # Stop time (min).
    tvec = unique(sort(c(TSTART, TSTOP, times[times > TSTART])))
    parms["VWELL"] = 0.0        # Volume of skin exposure well (mL). Setting
                                # this to zero creates a zero flux condition at
                                # the outer surface of the stratum corneum.
    parms = initParms(parms)
    out2 = run_model(model_name, tvec, Y0, parms)
    
    # Package results in a single data frame.
    out = rbind(out0[1:nrow(out0)-1, ], out1[1:nrow(out1)-1, ], out2)
    out = out[out[ , "time"] %in% times, ]
    
    # Return simulation output.
    return(out)
}


# Initialize parameters with human default values.
q = get_human_parms()

# Modify relevant parameters.
q["AEXP"] = 20.0        # Area (cm^2) of exposed skin *during* skin exposure.
q["DSC"] = 9.09e-8      # SC diffusion parameter (cm^2/min).
q["HSCJP8"] = 0.204     # SC to JP-8 partition coefficient.
q = initParms(q)

# The following are also model parameters for which we will compute model
# sensitivities, but are not classified as "parameters" in the model
# specification file.
q["exp_time"] = 36.0    # Total duration (min) of skin exposure.
q["RIV"] = 1.34         # Rate of background (intravenous) exposure (nmol/min).
q["CNJP8"] = 0.003      # Concentration of naphthalene in JP-8 (g/mL).

# Run a simulation for the parameter values in q.
out = run_sim(q)

# Plot blood concentration vs. time.
tout = out[, "time"] / 60.0                 # Simulation times (h).
CAB_ug_per_L = out[ , "CAB_UG"] * 1000.0    # Blood concentrations (ug/L).
plot(tout, CAB_ug_per_L, type="l", lwd=2, lty=1, col=vcol[1],
     xlab="Time (h)", ylab="Concentration in Blood (ug/L)")

# Plot skin concentration profile at 0.5 h after start of exposure.
tout = out[, "time"] / 60.0                 # Simulation times (h).
depth = q["TSC"] * 1000 * seq(from=0, to=1, by=0.1)  # Depth (um).
tidx = which(tout == 0.5)[1]
conc_sc = out[tidx, c("CSC00", "CSC01", "CSC02", "CSC03", "CSC04", "CSC05",
                      "CSC06", "CSC07", "CSC08", "CSC09", "CSC10")]
plot(depth, conc_sc, type="l", lwd=2, lty=1, col=vcol[1],
     xlab=expression(paste("Depth (", mu, "m)")),
     ylab="Concentration (nmol/mL)")

# Plot skin concentration profiles at various times.
file_name = "./Output/skin_conc_profile.tif"
tiff(file_name, units="in", width=5, height=4, res=300)
par(mar=c(5, 6, 4, 1) + 0.1)
plot(NULL, NULL, xlim=c(min(depth), max(depth)), ylim=c(0, 5000),
     xlab=expression(paste("Depth (", mu, "m)")),
     ylab="Concentration (nmol/mL)")
tplot = c(0.1, 0.5, 1.0, 2.0, 5.0)  # Times (h) for concentration profiles.
tlegend = vector(mode="character", length=length(tplot))
for (idx in seq(length(tplot))) {
    tidx = which(tout == tplot[idx])[1]
    tlegend[idx] = sprintf("%.1f h", tplot[idx])
    conc_sc = out[tidx, c("CSC00", "CSC01", "CSC02", "CSC03", "CSC04", "CSC05",
                          "CSC06", "CSC07", "CSC08", "CSC09", "CSC10")]
    lines(depth, conc_sc, lty=1, lwd=2, col=vcol[idx])
}
legend("topright", legend=tlegend, lty=1, lwd=2, col=vcol,
       border=NA, box.lty=1, cex=0.9)
dev.off()


# Plot skin concentration profiles at more times.
vcol = c("#440154FF", "#482173FF", "#433E85FF", "#38598CFF", "#2D708EFF",
         "#25858EFF", "#1E9B8AFF", "#2BB07FFF", "#51C56AFF", "#85D54AFF",
         "#C2DF23FF", "#FDE725FF")
file_name = "./Output/skin_conc_profile_more.tif"
tiff(file_name, units="in", width=6, height=5, res=300)
par(mar=c(5, 6, 4, 8) + 0.1, xpd=TRUE)
plot(NULL, NULL, xlim=c(min(depth), max(depth)), ylim=c(0, 5000),
     xlab=expression(paste("Depth (", mu, "m)")),
     ylab="Concentration (nmol/mL)")
tplot = c(seq(from=0.1, to=1.0, by=0.1), 2.0, 5.0)
tlegend = vector(mode="character", length=length(tplot))
for (idx in seq(length(tplot))) {
    tidx = which(tout == tplot[idx])[1]
    tlegend[idx] = sprintf("%.1f h", tplot[idx])
    if (tplot[idx] == 0.6) {  # If looking for the time when JP-8 is removed ...
        tidx = tidx - 1       # ... use the time point just before this occurs.
    }
    conc_sc = out[tidx, c("CSC00", "CSC01", "CSC02", "CSC03", "CSC04", "CSC05",
                          "CSC06", "CSC07", "CSC08", "CSC09", "CSC10")]
    lines(depth, conc_sc, lty=1, lwd=2, col=vcol[idx])
}
legend("right", inset=c(-0.4, 0), legend=tlegend, lty=1, lwd=2, col=vcol,
       border=NA, box.lty=1, cex=0.9)
dev.off()
