#------------------------------------------------------------------------------
# skin_exp_2c_fit_indiv.R
#
# Author: Dustin Kapraun, US EPA, May 2019
#
# This script calculates an optimal set of parameters for the model
# "naph_pbtk_2c" for 10 subjects from the controlled JP8 skin exposure study
# of Kim et al. (2006). The optimization is performed using data from each of
# the 10 subjects independently. In the optimization three parameters are
# allowed to vary for each subject:
#   (1) RIV, the "background" intravenous dosing rate (nmol/min);
#   (2) DSC, the diffusivity of the stratum corneum (cm^2/min); and
#   (3) HSCJP8, the SC to JP-8 partition coefficient.
# Note that the optimization is performed for multiple sets of initial
# parameter values for each subject. The results are stored in a data frame,
# which is then saved to an RDS file in the "Output" directory. Running this
# script may require a long time depending on the user's system. (The script
# required approximately 30 minutes on a DELL Latitude E7270 with an Intel Core
# i5-6300U 2.4 GHz CPU running Windows 10 Enterprise.) Results obtained by the
# authors are saved as a data frame in "Output/df_param_2c_indiv.rds".
#
# References:
# D Kim, ME Andersen, LA Nylander-French (2006), "A dermatotoxicokinetic model
# of human exposures to jet fuel," Toxicological Sciences 93(1), 22-23.
#------------------------------------------------------------------------------

# Set working directory to the directory containing this file.
script_dir = dirname(sys.frame(1)$ofile)
setwd(script_dir)

# Load the model "naph_pbtk_pde".
source("init_naph_pbtk_2c.R")

# Load the function "skin_exp_sim".
source("skin_exp_sim.R")

# Save some colors from the viridis palette. This vector of color-identification
# strings was generated by calling the function "viridis" from the package
# "viridis" with the argument "5"; i.e., "viridis(5)".
vcol = c("#440154FF", "#3B528BFF", "#21908CFF", "#5DC863FF", "#FDE725FF")

# Define a cost function.
cost_fun <- function(theta, t_data, c_data, BWinit, exp_time, parms) {
    # Unpack parameter vector.
    RIV = theta[1]
    DSC = theta[2]
    HSCJP8 = theta[3]

    # Get the molar mass of naphthalene (g/mol).
    MW1 = parms[["MW1"]]

    # Obtain modeled concentrations.
    # out = run_sim(t_data, BWinit, exp_time, RIV, DSC, HSCJP8, parms)
    out = skin_exp_sim(model_name, t_data, BWinit, exp_time, RIV, DSC, HSCJP8,
                       parms)
    c_model = out[ , "CVB"] * MW1
    
    # Calculate sum of squared errors.
    SE = (c_model - c_data) ** 2
    SSE = sum(SE)
    
    # Return sum of squared errors.
    return(SSE)
}

# Load data from Kim et al. (2006).
df_blood = read.csv("./Data/kim_blood_data.csv", row.names=1,
                    check.names=FALSE)
df_subject = read.csv("./Data/kim_subject_data.csv", row.names=1,
                      check.names=FALSE)

# Load human model parameters into "parms".
parms = get_human_parms()

# Dermal physicochemical parameters (McCarley & Bunge, 2001).
DSC = parms[["TSC"]] / 60 * exp(-2.8 - 0.006 * parms[["MW1"]]) # Eq. 28.
parms["DSC"] = DSC     # Diffusivity across SC (cm^2/min).
DVE = (7.1e-6) * 60 * (parms[["MW1"]]) ** 0.5  # Eq. 29.
parms["DVE"] = DVE     # Diffusivity through VE (cm^2/min).


# Set inititial parameter Values.
RIV_init_vec = c(1.0, 1.5, 2.0)       # IV dose (nmol/min).
DSC_init_vec = c(1e-6, 1e-7, 1e-8)    # Diffusivity through SC (cm^2/min).
HSCJP8_init_vec = c(0.05, 0.2, 0.5)   # Partition coefficient.

# Get all unique subject identifiers.
subject_ids = unique(df_blood[["Subject"]])

# Allocate array to contain optimal parameters for all subjects.
df_param = data.frame(Subject=integer(),
                      RIV=double(),
                      DSC=double(),
                      HSCJP8=double(),
                      cost=double())

# Save the start time.
time0 = Sys.time()

# For each subject, find model parameters (RIV, DSC, and HSCJP8) that fit the
# data.
for (subject_id in subject_ids) {
    # --- Acquire subject data. ---
    # Body mass (g) for this subject (Kim et al., 2006).
    BWinit = df_subject[subject_id, "Body Mass (kg)"] * 1000

    # Exposure time (min) for this subject. This is the amount of time that
    # the JP-8 was in the wells on the subject's forearm (Kim et al., 2006).
    exp_time = df_subject[subject_id, "Exposure Duration (min)"]

    # Blood concentration observation times (min).
    t_data = df_blood[df_blood["Subject"] == subject_id, "Time (min)"]
    
    # Observed blood concentations (ng/mL = ug/L).
    c_data = df_blood[df_blood["Subject"] == subject_id,
                      "Concentration (ng/mL)"]
    
    # Set initial parameter vector and run optimization.
    cost_opt = Inf
    theta_opt = c(0, 0, 0)
    for (RIV_idx in seq(length(RIV_init_vec))) {
        RIV = RIV_init_vec[RIV_idx]
        for (DSC_idx in seq(length(DSC_init_vec))) {
            DSC = DSC_init_vec[DSC_idx]
            for (HSCJP8_idx in seq(length(HSCJP8_init_vec))) {
                HSCJP8 = HSCJP8_init_vec[HSCJP8_idx]
                theta_init = c(RIV, DSC, HSCJP8)
                
                # Get cost for initial parameter vector.
                cost = cost_fun(theta_init, t_data, c_data, BWinit, exp_time,
                                parms=parms)
                cat("\nResults for Subject ", subject_id, ", Initial Theta ",
                    RIV_idx-1, DSC_idx-1, HSCJP8_idx-1, ".\n", sep="")
                cat("   Cost for default parameter is ", cost, ".\n", sep="")
                cat("   RIV =", theta_init[1], "DSC =", theta_init[2],
                    "HSCJP8 =", theta_init[3], "\n")
                
                # Optimize parameters.
                opt_res = optim(theta_init, fn=cost_fun, gr=NULL,
                                t_data, c_data, BWinit, exp_time, parms,
                                method="L-BFGS-B", lower=c(1.0, 1.5e-9, 0.001),
                                upper=c(2.0, 1.5e-3, 3.0),
                                control=list(parscale=theta_init))
                cat("   Optimization terminated with status ",
                    opt_res$convergence, ".\n", sep="")
                if (opt_res$convergence > 0) {
                  cat("   ", opt_res$message, "\n", sep="")
                } else {
                  cat("   Cost for optimal parameter is ", opt_res$value, ".\n", sep="")
                  cat("   RIV =", opt_res$par[1], "DSC =", opt_res$par[2],
                      "HSCJP8 =", opt_res$par[3], "\n")
                  
                  # If cost has been decreased, save optimal paramter vector.
                  if (opt_res$value < cost_opt) {
                      cost_opt = opt_res$value
                      theta_opt = opt_res$par
                  }
                }
            }
        }
    }
    
    # Save optimal parameter values.
    RIV = theta_opt[1]        # IV dose (nmol/min).
    DSC = theta_opt[2]        # Diffusivity across SC (cm^2/min).
    HSCJP8 = theta_opt[3]     # SC to JP-8 partition coefficient.
    df_param[nrow(df_param)+1, ] = c(subject_id, RIV, DSC, HSCJP8, cost_opt)

    # Run simulation with optimal parameter values.
    t_sim = seq(from=0, to=300, by=1)
    out = skin_exp_sim(model_name, t_sim, BWinit, exp_time, RIV, DSC, HSCJP8,
                       parms)

    # Extract results
    t_sim = out[ , "time"]
    CVB_sim = out[ , "CVB"]  # Concentration in venous blood (nmol/mL).
    CVB_sim = CVB_sim * parms[["MW1"]]  # ... (ug/L).
    # AWELL_sim = out[ , "AWELL"] # Amount in dermal exposure well (nmol).
    # AWELL_sim = AWELL_sim * MW1 / 1e6      # ... (mg)
    # AINa_sim = out[ , "AIN"] # Amount in organism plus amount metabolized (nmol).
    # AINa_sim = AINa_sim * MW1 / 1e6     # ... (mg)
    # AIN_sim = out[ , "AIN"]  # Net amount that has entered organism (nmol).
    # AIN_sim = AIN_sim * MW1 / 1e6       # ... (mg)
    # CWELL_sim = out[ , "CWELL"] # Concentration in dermal exposure well (nmol/mL).
    # CWELL_sim = CWELL_sim * MW1 / 1e6      # ... (mg/mL)

    # Generate a figure that shows naphthalene blood concentration vs. time.
    ymin = min(min(CVB_sim), min(c_data)) * 0.8
    ymax = max(max(CVB_sim), max(c_data)) * 1.1
    plot(t_sim, CVB_sim, type="l", lwd=2, lty=1, col=vcol[1],
         xlab="Time (min)", ylab="Concentration in Blood (ug/L)",
         ylim=c(ymin, ymax))
    points(t_data, c_data, pch=19, col=vcol[2], cex=1.0)

    # # Generate a figure that shows amount in well vs. time.
    # plot(t_sim, AWELL_sim, type="l", lwd=2, lty=1, col="blue",
    #      xlab="Time (min)", ylab="Amount in Well (mg)")

    # # Generate a figure that shows concentration in well vs. time.
    # plot(t_sim, CWELL_sim, type="l", lwd=2, lty=1, col="blue",
    #      xlab="Time (min)", ylab="Concentration in Well (mg/mL)")

    # # Generate a figure that shows amount in organism plus total amount metabolized
    # # vs. time.
    # plot(t_sim, AINa_sim, type="l", lwd=2, lty=1, col="blue",
    #      xlab="Time (min)", ylab="Amount in Organism + Amount Metabolized (mg)")
    # lines(t_sim, AIN_sim, type="l", lwd=2, lty=2, col="red")
}

# Save optimal parameter values.
saveRDS(df_param, file="./Output/df_param_2c_indiv.rds")

# Print cumulative computational time.
comp_time = difftime(Sys.time(), time0, units="mins")
cat("\nTime required to run this script:", comp_time[[1]], "minutes.\n")