#------------------------------------------------------------------------------
# skin_exp_2c_plot_indiv.R
#
# Author: Dustin Kapraun, US EPA, May 2019
#
# This script uses results from "skin_exp_2c_fit_indiv.R" that are saved in
# the file "df_param_2c_indiv.rds" in the "Output" directory. An optimal set
# of parameters for the model "naph_pbtk_2c" for 10 subjects from the
# controlled JP8 skin exposure study of Kim et al. (2006) is used to plot model
# simulated blood concentrations along with observed blood concentrations for
# each of the 10 subjects. Plot figures are saved to the "Output" directory in
# files with names of the form "skin_exp_2c_subjectXX_fit_indiv.tif" where
# "XX" is a number between 01 and 10 indicating the subject.
#
# Summary statistical information for the optimal parameters, including mean
# and coefficient of variation, are also calculated and printed to the display
# (standard output).
#
# Only the following three model parameters are considered:
#   (1) RIV, the "background" intravenous dosing rate (nmol/min);
#   (2) DSC, the diffusivity of the stratum corneum (cm^2/min); and
#   (3) HSCJP8, the SC to JP-8 partition coefficient.
#
# References:
# D Kim, ME Andersen, LA Nylander-French (2006), "A dermatotoxicokinetic model
# of human exposures to jet fuel," Toxicological Sciences 93(1), 22-23.
#------------------------------------------------------------------------------

# Set working directory to the directory containing this file.
script_dir = dirname(sys.frame(1)$ofile)
setwd(script_dir)

# Load the model "naph_pbtk_2c".
source("init_naph_pbtk_2c.R")

# Load the function "skin_exp_sim".
source("skin_exp_sim.R")

# Save some colors from the viridis palette. This vector of color-identification
# strings was generated by calling the function "viridis" from the package
# "viridis" with the argument "5"; i.e., "viridis(5)".
vcol = c("#440154FF", "#3B528BFF", "#21908CFF", "#5DC863FF", "#FDE725FF")

# Load data from Kim et al. (2006).
df_blood = read.csv("./Data/kim_blood_data.csv", row.names=1,
                    check.names=FALSE)
df_subject = read.csv("./Data/kim_subject_data.csv", row.names=1,
                      check.names=FALSE)

# Get number of subjects.
num_subjects = length(unique(df_blood[["Subject"]]))

# Load human model parameters into "parms".
parms = get_human_parms()

# Get all unique subject identifiers.
subject_ids = unique(df_blood[["Subject"]])

# Load the data frame containing optimal parameter values for each subject.
df_param = readRDS("./Output/df_param_2c_indiv.rds")

# Simulation time points to use for plotting.
t_sim = seq(from=0, to=300, by=1)

# Create a data frame to store plot points.
df_plot = data.frame(matrix(NA, nrow=length(t_sim), ncol=length(subject_ids)+1))
colnames(df_plot) = c("t_sim", subject_ids)
df_plot[ , "t_sim"] = t_sim

# Generate a plot for each subject using the optimal parameter values.
for (sidx in seq(num_subjects)) {
    # Get parameters for this subject.
    RIV = df_param[sidx, "RIV"]
    DSC = df_param[sidx, "DSC"]
    HSCJP8 = df_param[sidx, "HSCJP8"]

    # Get relevant data (Kim et al., 2006) for this subject.
    subject_id = df_param[sidx, "Subject"]

    # Body mass (g) for this subject.
    BWinit = df_subject[subject_id, "Body Mass (kg)"] * 1000

    # Exposure time (min) for this subject. This is the amount of time that
    # the JP-8 was in the wells on the subject's forearm.
    exp_time = df_subject[subject_id, "Exposure Duration (min)"]

    # Blood concentration observation times (min).
    t_data = df_blood[df_blood["Subject"] == subject_id, "Time (min)"]

    # Observed blood concentations (ng/mL = ug/L).
    c_data = df_blood[df_blood["Subject"] == subject_id,
                      "Concentration (ng/mL)"]

    # Print parameter values.
    # cat("Subject ", subject_id, ":\n", sep="")
    # cat("  RIV = ", RIV, "DSC = ", DSC, "HSCJP8 = ", HSCJP8, "\n")

    # Run simulation with optimal parameter values.
    out = skin_exp_sim(model_name, t_sim, BWinit, exp_time, RIV, DSC, HSCJP8,
                       parms)

    # Extract results.
    CVB_sim = out[ , "CVB"]  # Concentration in venous blood (nmol/mL).
    CVB_sim = CVB_sim * parms[["MW1"]]             # ... (ug/L).
    
    # Save results to plot data frame.
    df_plot[ , sidx+1] = CVB_sim

    # Generate a figure that shows naphthalene blood concentration vs. time.
    file_name = paste0("./Output/skin_exp_2C_subject", 
                       formatC(subject_id, digits=0, width=2, format="d",
                               flag="0"),
                       "_fit_indiv.tif")
    tiff(file_name, units="in", width=4, height=4, res=300)
    ymin = min(min(CVB_sim), min(c_data)) * 0.8
    ymax = max(max(CVB_sim), max(c_data)) * 1.1
    plot(t_sim, CVB_sim, type="l", lwd=2, lty=1, col=vcol[1],
         xlab="Time (min)", ylab="Concentration in Blood (ug/L)",
         ylim=c(ymin, ymax))
    points(t_data, c_data, pch=19, col=vcol[2], cex=1.0)
    dev.off()
}

# Save plot information.
saveRDS(df_plot, file="./Output/df_plot_2c_indiv.rds")

# Calculate mean and coefficient of variation for each parameter.
df_mean = apply(df_param, 2, mean)
df_cv = apply(df_param, 2, sd) / df_mean * 100

# Calculate mean and coefficient of variation for each parameter when excluding
# Subject 1.
df_mean_x1 = apply(df_param[2:10, ], 2, mean)
df_cv_x1 = apply(df_param[2:10, ], 2, sd) / df_mean_x1 * 100

# Display data frame.
update_cols = c("Subject", "RIV", "DSC", "HSCJP8")
df_param[11, update_cols] = c("Mean", df_mean[c("RIV", "DSC", "HSCJP8")])
df_param[12, update_cols] = c("CV", df_cv[c("RIV", "DSC", "HSCJP8")])
df_param[13, update_cols] = c("Mean_x1", df_mean_x1[c("RIV", "DSC", "HSCJP8")])
df_param[14, update_cols] = c("CV_x1", df_cv_x1[c("RIV", "DSC", "HSCJP8")])

print(df_param)