#------------------------------------------------------------------------------
# skin_exp_pde_error_all.R
#
# Author: Dustin Kapraun, US EPA, July 2020
#
# This script uses results from "skin_exp_pde_fit_all.R" that are saved in the
# file "df_param_pde_all.rds" in the "Output" directory. An optimal set of parameters
# for 10 subjects from the controlled JP8 dermal exposure study of Kim et al.
# (2006) is used to calculate model-predicted blood concentrations for
# comparison with observed blood concentrations for each of the 10 subjects.
#
# Percent differences for raw concentrations and percent errors for difference-
# from-steady-state concentrations are calculated and summary information is
# printed to the terminal.
#
# Only the following three model parameters are considered:
#   (1) RIV, the "background" intravenous dosing rate (nmol/min);
#   (2) DSC, the diffusivity of the stratum corneum (cm^2/min); and
#   (3) HSCJP8, the SC to JP-8 partition coefficient.
# In the optimization, we assume that HSCJP8 is identical for all subjects.
#
# References:
# D Kim, ME Andersen, LA Nylander-French (2006), "A dermatotoxicokinetic model
# of human exposures to jet fuel," Toxicological Sciences 93(1), 22-23.
#------------------------------------------------------------------------------

# Set working directory to the directory containing this file.
script_dir = dirname(sys.frame(1)$ofile)
setwd(script_dir)

# Load the model "naph_pbtk_pde".
source("init_naph_pbtk_pde.R")

# Load the function "skin_exp_sim".
source("skin_exp_sim.R")

# Save some colors from the viridis palette. This vector of color-identification
# strings was generated by calling the function "viridis" from the package
# "viridis" with the argument "5"; i.e., "viridis(5)".
vcol = c("#440154FF", "#3B528BFF", "#21908CFF", "#5DC863FF", "#FDE725FF")

# Load data from Kim et al. (2006).
df_blood = read.csv("./Data/kim_blood_data.csv", row.names=1,
                    check.names=FALSE)
df_subject = read.csv("./Data/kim_subject_data.csv", row.names=1,
                      check.names=FALSE)

# Get number of subjects.
num_subjects = length(unique(df_blood[["Subject"]]))

# Load human model parameters into "parms".
parms = get_human_parms()

# Get all unique subject identifiers.
subject_ids = unique(df_blood[["Subject"]])

# Load the data frame containing optimal parameter values for each subject.
df_paramx = readRDS("./Output/df_param_pde_all.rds")

# Create a vectors to store percent differences between predicted and observed
# raw concentrations.
pdiff = c()

# Create a vector to store percent errors between predicted and observed
# difference-from-steady-state concentrations.
perr = c()

# Generate a plot for each subject using the optimal parameter values.
for (sidx in seq(num_subjects)) {
    # Get parameters for this subject.
    RIV = df_paramx[sidx, "RIV"]
    DSC = df_paramx[sidx, "DSC"]
    HSCJP8 = df_paramx[sidx, "HSCJP8"]

    # Get relevant data (Kim et al., 2006) for this subject.
    subject_id = df_paramx[sidx, "Subject"]

    # Body mass (g) for this subject.
    BWinit = df_subject[subject_id, "Body Mass (kg)"] * 1000

    # Exposure time (min) for this subject. This is the amount of time that
    # the JP-8 was in the wells on the subject's forearm.
    exp_time = df_subject[subject_id, "Exposure Duration (min)"]

    # Blood concentration observation times (min).
    t_data = df_blood[df_blood["Subject"] == subject_id, "Time (min)"]

    # Observed blood concentations (ng/mL = ug/L).
    c_data = df_blood[df_blood["Subject"] == subject_id,
                      "Concentration (ng/mL)"]

    # Print parameter values.
    # cat("Subject ", subject_id, ":\n", sep="")
    # cat("  RIV = ", RIV, "DSC = ", DSC, "HSCJP8 = ", HSCJP8, "\n")

    # Run simulation with optimal parameter values.
    out = skin_exp_sim(model_name, t_data, BWinit, exp_time, RIV, DSC, HSCJP8,
                       parms)

    # Extract results.
    CVB_sim = out[ , "CVB"]  # Concentration in venous blood (nmol/mL).
    CVB_sim = CVB_sim * parms[["MW1"]]             # ... (ug/L).

    # Save percent differences between predicted and observed raw
    # concentrations.
    for (idx in seq(length(t_data))) {
        pdiff[length(pdiff)+1] = (CVB_sim[idx] - c_data[idx]) * 2 /
            (CVB_sim[idx] + c_data[idx])
    }
    
    # Save percent errors between predicted and observed difference-from-
    # steady-state concentrations.
    for (idx in seq(from=1, to=length(t_data))) {
        c_pred = CVB_sim[idx] - CVB_sim[1]
        c_obs = c_data[idx] - CVB_sim[1]
        perr[length(perr)+1] = (c_pred - c_obs) / c_obs
    }

    # # Generate a figure that shows naphthalene blood concentration vs. time.
    # file_name = paste0("./Output/skin_exp_pde_error_subject", 
    #                    formatC(subject_id, digits=0, width=2, format="d",
    #                            flag="0"),
    #                    "_fit_all.tif")
    # tiff(file_name, units="in", width=4, height=4, res=300)
    # ymin = min(min(CVB_sim), min(c_data)) * 0.8
    # ymax = max(max(CVB_sim), max(c_data)) * 1.1
    # plot(t_data, CVB_sim, type="p", pch=19, col=vcol[1], cex=1.0,
    #      xlab="Time (min)", ylab="Concentration in Blood (ug/L)",
    #      ylim=c(ymin, ymax))
    # points(t_data, c_data, pch=19, col=vcol[2], cex=1.0)
    # dev.off()
}

# Print statements about maximum and average percent difference between
# predicted and observed raw concentrations.
cat("Maximum absolute percent difference in raw concentrations is ",
    max(abs(pdiff)) * 100, "%.\n", sep="")
cat("Average absolute percent difference in raw concentrations is ",
    mean(abs(pdiff)) * 100, "%.\n", sep="")

# Print statements about percent errors between predicted and observed
# difference-from-steady-state concentrations.
cat("Median absolute percent error in difference-from-steady-state ",
    "concentrations is ", median(abs(perr)) * 100,
    "%.\n", sep="")
cat(sum(abs(perr) <= 2.0) / length(perr) * 100, "% of difference-from-",
    "steady-state predictions are within a factor of two of observations.\n",
    sep="")