#------------------------------------------------------------------------------
# skin_exp_pde_fit_all.R
#
# Author: Dustin Kapraun, US EPA, May 2019
#
# This script calculates an optimal set of parameters for the model
# "naph_pbtk_pde" for 10 subjects from the from the controlled JP8 skin
# exposure study of Kim et al. (2006). The optimization is performed using data
# from all 10 subjects simultaneously rather than optimizing independently for
# each subject. In the optimization three parameters are allowed to vary for
# each subject:
#   (1) RIV, the "background" intravenous dosing rate (nmol/min);
#   (2) DSC, the diffusivity of the stratum corneum (cm^2/min); and
#   (3) HSCJP8, the SC to JP-8 partition coefficient.
# However, we assume that HSCJP8 is identical for all subjects; thus, a total
# of 21 parameters are estimated simultaneously:
#   (1-10) RIV for 10 subjects;
#   (11-20) DSC for 10 subjects; and
#   (21) HSCJP8 for all subjects.
# Note that the optimization is performed for 10 randomly selected sets of
# initial parameter values. The results are stored in a data frame, which is
# then saved to an RDS file in the "Output" directory. Running this script may
# require a long time depending on the user's system. (The script required
# approximately 5 hours on a DELL Precision T7610 with 16 Intel Xeon E5-2650
# (v2) 2.6 GHz CPUs running Red Hat Enterprise Linux Workstation 7.7.) Results
# obtained by the authors are saved as a data frame in the "Output" directory in
# the file "df_param_pde_all.rds".
#
# References:
# D Kim, ME Andersen, LA Nylander-French (2006), "A dermatotoxicokinetic model
# of human exposures to jet fuel," Toxicological Sciences 93(1), 22-23.
#------------------------------------------------------------------------------

# Set working directory to the directory containing this file.
script_dir = dirname(sys.frame(1)$ofile)
setwd(script_dir)

# Load the model "naph_pbtk_pde".
source("init_naph_pbtk_pde.R")

# Load the function "skin_exp_sim".
source("skin_exp_sim.R")

# Save some colors from the viridis palette. This vector of color-identification
# strings was generated by calling the function "viridis" from the package
# "viridis" with the argument "5"; i.e., "viridis(5)".
vcol = c("#440154FF", "#3B528BFF", "#21908CFF", "#5DC863FF", "#FDE725FF")

# Define cost function.
cost_fun <- function(theta, df_subject, df_blood, parms) {
    # Unpack parameter vector.
    RIV_subject = theta[1:10]
    DSC_subject = theta[11:20]
    HSCJP8 = theta[21]

    # Get the molar mass of naphthalene (g/mol).
    MW1 = parms[["MW1"]]

    # Initialize sum of squared errors (SSE) to zero.
    SSE = 0

    # Get all unique subject identifiers.
    subject_ids = unique(df_blood[["Subject"]])

    # For each subject, calculate model-predicted blood concentrations and
    # compute the SSE contribution.
    for (idx in seq(length(subject_ids))) {
        # Get values of RIV and DSC to use for this subject.
        RIV = RIV_subject[idx]
        DSC = DSC_subject[idx]

        # Get subject ID.
        subject_id = subject_ids[idx]

        # Get relevant data (Kim et al., 2006) for this subject.
        # Body mass (g) for this subject.
        BWinit = df_subject[subject_id, "Body Mass (kg)"] * 1000

        # Exposure time (min) for this subject. This is the amount of time that
        # the JP-8 was in the wells on the subject's forearm.
        exp_time = df_subject[subject_id, "Exposure Duration (min)"]

        # Blood concentration observation times (min).
        t_data = df_blood[df_blood["Subject"] == subject_id, "Time (min)"]

        # Observed blood concentations (ng/mL = ug/L).
        c_data = df_blood[df_blood["Subject"] == subject_id,
                          "Concentration (ng/mL)"]

        # Calculate model-predicted blood concentrations.
        # out = run_sim(t_data, BWinit, exp_time, RIV, DSC, HSCJP8, parms)
        out = skin_exp_sim(model_name, t_data, BWinit, exp_time, RIV, DSC,
                           HSCJP8, parms)
        c_model = out[ , "CVB"] * MW1

        # Calculate sum of squared errors.
        SE = (c_model - c_data) ** 2
        SSE = SSE + sum(SE)
    }

    # Return sum of squared errors.
    return(SSE)
}

# Load data from Kim et al. (2006).
df_blood = read.csv("./Data/kim_blood_data.csv", row.names=1,
                    check.names=FALSE)
df_subject = read.csv("./Data/kim_subject_data.csv", row.names=1,
                      check.names=FALSE)

# Get number of subjects.
num_subjects = length(unique(df_blood[["Subject"]]))

# Load human model parameters into "parms".
parms = get_human_parms()

# # Set initial parameter values.
# RIV_init = 1.4       # IV dose (nmol/min).
# DSC_init = 1.0e-7    # Diffusivity through SC (cm^2/min).
# HSCJP8_init = 0.175  # SC-to-JP8 partition coefficient.
# theta_init = c(rep(RIV_init, num_subjects), rep(DSC_init, num_subjects),
#                HSCJP8_init)
# theta_init = c(1.34, 1.69, 1.45, 1.32, 1.14, 1.70, 1.16, 1.42, 1.13, 1.41, 
#                2.34e-5, 6.96e-8, 2.64e-8, 1.04e-7, 1.18e-7, 1.09e-7, 8.04e-8,
#                3.38e-7, 1.26e-7, 2.31e-7, 0.175)

# Create a data frame to store optimal parameter values.
df_param = data.frame(matrix(NA, nrow=10, ncol=21))
df_param["Cost"] = Inf

# Create a data frame to store initial parameter values.
df_param_init = df_param

# Set a seed for random number generator (for reproducibility).
set.seed(2019)

# Save the start time.
time0 = Sys.time()

for (idx in seq(nrow(df_param))) {
    # Generate and save initial parameter vector.
    theta_init = c(runif(num_subjects, 0.5, 2.5),
                   runif(num_subjects, 1e-9, 1e-5),
                   runif(1, 0.1, 0.5))
    df_param_init[idx, 1:length(theta_init)] = theta_init

    # Print cost for initial parameter vector.
    cost = cost_fun(theta_init, df_subject, df_blood, parms=parms)
    cat("Cost for initial parameter ", idx, " is ", cost, ".\n", sep="")
    df_param_init[idx, "Cost"] = cost
    
    # Optimize parameters.
    opt_res = optim(theta_init, fn=cost_fun, gr=NULL,
                    df_subject, df_blood, parms,
                    method="L-BFGS-B", lower=rep(0, length(theta_init)),
                    control=list(parscale=theta_init, maxit=500))

    # Print optimization results.
    cat("Optimization ", idx, " terminated with status ", opt_res$convergence,
        ".\n", sep="")
    if (opt_res$convergence > 0) {
        cat("   ", opt_res$message, "\n", sep="")
    } else {
        cat("Cost for optimal parameter ", idx, " is ", opt_res$value, ".\n",
            sep="")
        df_param[idx, 1:length(theta_init)] = opt_res$par
        df_param[idx, "Cost"] = opt_res$value
    }
    
    # Print cumulative computational time.
    comp_time = difftime(Sys.time(), time0, units="mins")
    cat("Time required so far:", comp_time[[1]], "minutes.\n")
}

# Save initial parameters and optimization results.
saveRDS(df_param_init, file="./Output/df_param_pde_init_all.rds")
saveRDS(df_param, file="./Output/df_param_pde_all.rds")

# Get all unique subject identifiers.
subject_ids = unique(df_blood[["Subject"]])

# Create a data frame to store optimal parameter values for each subject.
df_paramx = data.frame(Subject=integer(),
                       RIV=double(),
                       DSC=double(),
                       HSCJP8=double())

# Get index of the optimal parameter set with the lowest cost.
lowest_cost = which(df_param["Cost"] == min(df_param["Cost"]))

# Generate a plot for each subject using the optimal parameter set with lowest
# cost.
for (sidx in seq(num_subjects)) {
    # Get parameters for this subject.
    RIV = df_param[lowest_cost[1], sidx]
    DSC = df_param[lowest_cost[1], sidx+10]
    HSCJP8 = df_param[lowest_cost[1], 21]

    # Get relevant data (Kim et al., 2006) for this subject.
    subject_id = subject_ids[sidx]
    df_paramx[sidx, ] = c(subject_id, RIV, DSC, HSCJP8)

    # Body mass (g) for this subject.
    BWinit = df_subject[subject_id, "Body Mass (kg)"] * 1000

    # Exposure time (min) for this subject. This is the amount of time that
    # the JP-8 was in the wells on the subject's forearm.
    exp_time = df_subject[subject_id, "Exposure Duration (min)"]

    # Blood concentration observation times (min).
    t_data = df_blood[df_blood["Subject"] == subject_id, "Time (min)"]

    # Observed blood concentations (ng/mL = ug/L).
    c_data = df_blood[df_blood["Subject"] == subject_id,
                      "Concentration (ng/mL)"]

    # Print parameter values.
    cat("Subject ", subject_id, ":\n", sep="")
    cat("  RIV = ", RIV, "DSC = ", DSC, "HSCJP8 = ", HSCJP8, "\n")

    # Run simulation with optimal parameter values.
    t_sim = seq(from=0, to=300, by=1)
    # out = run_sim(t_sim, BWinit, exp_time, RIV, DSC, HSCJP8, parms)
    out = skin_exp_sim(model_name, t_sim, BWinit, exp_time, RIV, DSC, HSCJP8,
                       parms)
    # Extract results
    t_sim = out[ , "time"]
    CVB_sim = out[ , "CVB"]  # Concentration in venous blood (nmol/mL).
    CVB_sim = CVB_sim * parms[["MW1"]]             # ... (ug/L).

    # Generate a figure that shows naphthalene blood concentration vs. time.
    ymin = min(min(CVB_sim), min(c_data)) * 0.8
    ymax = max(max(CVB_sim), max(c_data)) * 1.1
    plot(t_sim, CVB_sim, type="l", lwd=2, lty=1, col=vcol[1],
         xlab="Time (min)", ylab="Concentration in Blood (ug/L)",
         ylim=c(ymin, ymax))
    points(t_data, c_data, pch=19, col=vcol[2], cex=1.0)
}

# Save optimization results.
saveRDS(df_paramx, file="./Output/df_param_pde_all.rds")

# Print cumulative computational time.
comp_time = difftime(Sys.time(), time0, units="mins")
cat("\nTime required to run this script:", comp_time[[1]], "minutes.\n")
