#------------------------------------------------------------------------------
# skin_exp_pde_fit_indiv1.R
#
# Author: Dustin Kapraun, US EPA, May 2019
#
# This script calculates an optimal set of parameters for the model
# "naph_pbtk_pde" for 10 subjects from the controlled JP8 skin exposure study
# of Kim et al. (2006). The optimization is performed using data from each of
# the 10 subjects independently. In the optimization three parameters are
# allowed to vary for each subject:
#   (1) RIV, the "background" intravenous dosing rate (nmol/min);
#   (2) DSC, the diffusivity of the stratum corneum (cm^2/min); and
#   (3) HSCJP8, the SC to JP-8 partition coefficient.
# Note that the optimization is performed only for a single set of
# initial parameter values for each subject. An alternate version of this
# script uses multiple sets of initial parameter values for each subject.
# The results are stored in a data frame, which is then saved to an RDS file in
# the "Output" directory. Results obtained by the authors are saved as a data
# frame in the "Output" directory in the file "df_param_pde_indiv1.rds".
#
# References:
# D Kim, ME Andersen, LA Nylander-French (2006), "A dermatotoxicokinetic model
# of human exposures to jet fuel," Toxicological Sciences 93(1), 22-23.
#------------------------------------------------------------------------------

# Set working directory to the directory containing this file.
script_dir = dirname(sys.frame(1)$ofile)
setwd(script_dir)

# Load the model "naph_pbtk_pde".
source("init_naph_pbtk_pde.R")

# Load the function "skin_exp_sim".
source("skin_exp_sim.R")

# Save some colors from the viridis palette. This vector of color-identification
# strings was generated by calling the function "viridis" from the package
# "viridis" with the argument "5"; i.e., "viridis(5)".
vcol = c("#440154FF", "#3B528BFF", "#21908CFF", "#5DC863FF", "#FDE725FF")

# Define a cost function.
cost_fun <- function(theta, t_data, c_data, BWinit, exp_time, parms) {
    # Unpack parameter vector.
    RIV = theta[1]
    DSC = theta[2]
    HSCJP8 = theta[3]

    # Get the molar mass of naphthalene (g/mol).
    MW1 = parms[["MW1"]]

    # Obtain modeled concentrations.
    out = skin_exp_sim(model_name, t_data, BWinit, exp_time, RIV, DSC, HSCJP8,
                       parms)
    c_model = out[ , "CVB"] * MW1
    
    # Calculate sum of squared errors.
    SE = (c_model - c_data) ** 2
    SSE = sum(SE)
    
    # Return sum of squared errors.
    return(SSE)
}

# Load data from Kim et al. (2006).
df_blood = read.csv("./Data/kim_blood_data.csv", row.names=1,
                    check.names=FALSE)
df_subject = read.csv("./Data/kim_subject_data.csv", row.names=1,
                      check.names=FALSE)

# Load human model parameters into "parms".
parms = get_human_parms()

# Get all unique subject identifiers.
subject_ids = unique(df_blood[["Subject"]])

# Create a data frame to contain optimal parameters for all subjects.
df_param = data.frame(Subject=integer(),
                      RIV=double(),
                      DSC=double(),
                      HSCJP8=double(),
                      cost=double())

# Save the start time.
time0 = Sys.time()

# For each subject, find model parameters (RIV, HSCJP8, and DSC) that fit the
# data.
for (subject_id in subject_ids) {
    # Acquire subject data...
    # Body mass (g) for this subject (Kim et al., 2006).
    BWinit = df_subject[subject_id, "Body Mass (kg)"] * 1000

    # Exposure time (min) for this subject. This is the amount of time that
    # the JP-8 was in the wells on the subject's forearm (Kim et al., 2006).
    exp_time = df_subject[subject_id, "Exposure Duration (min)"]

    # Blood concentration observation times (min).
    t_data = df_blood[df_blood["Subject"] == subject_id, "Time (min)"]
    
    # Observed blood concentations (ng/mL = ug/L).
    c_data = df_blood[df_blood["Subject"] == subject_id,
                      "Concentration (ng/mL)"]

    # Set initial parameter values.
    MW1 = parms[["MW1"]]
    if (subject_id == 6) {
        IVDOSE = 1.1e-8             # IV dose (mg/min).
        RIV = IVDOSE * MW1 * 1e6    # IV dose (nmol/min).
        DSC = 1e-7                  # SC diffusion parameter (cm^2/min).
        HSCJP8 = 0.2                # SC to JP-8 partition coefficient.
    } else {
        IVDOSE = 1.3e-8             # IV dose (mg/min).
        RIV = IVDOSE * MW1 * 1e6    # IV dose (nmol/min).
        DSC = 1e-6                  # SC diffusion parameter (cm^2/min).
        HSCJP8 = 0.3                # SC to JP-8 partition coefficient.
    }
    theta_init = c(RIV, DSC, HSCJP8)
    cost = cost_fun(theta_init, t_data, c_data, BWinit, exp_time, parms=parms)
    cat("\nResults for Subject ", subject_id, ".\n", sep="")
    cat("   Cost for default parameter is ", cost, ".\n", sep="")

    # Optimize parameters.
    opt_res = optim(theta_init, fn=cost_fun, gr=NULL,
                    t_data, c_data, BWinit, exp_time, parms,
                    method="L-BFGS-B", lower=c(0, 0, 0),
                    control=list(parscale=theta_init))
    cat("   Optimization terminated with status ", opt_res$convergence, ".\n",
        sep="")
    if (opt_res$convergence > 0) {
        cat("   ", opt_res$message, sep="")
    } else {
        cat("   Cost for optimal parameter is ", opt_res$value, ".\n", sep="")
    }

    # Save optimal parameter values.
    RIV = opt_res$par[1]        # IV dose (nmol/min).
    DSC = opt_res$par[2]        # SC diffusion parameter (cm^2/min).
    HSCJP8 = opt_res$par[3]     # SC to JP-8 partition coefficient.
    cost_opt = opt_res$value    # Cost for optimal parameter.
    df_param[nrow(df_param)+1, ] = c(subject_id, RIV, DSC, HSCJP8, cost_opt)

    # Run simulation with optimal parameter values.
    t_sim = seq(from=0, to=300, by=1)
    out = skin_exp_sim(model_name, t_sim, BWinit, exp_time, RIV, DSC, HSCJP8,
                       parms)

    # Extract results
    t_sim = out[ , "time"]
    CVB_sim = out[ , "CVB"]  # Concentration in venous blood (nmol/mL).
    CVB_sim = CVB_sim * MW1  # ... (ug/L).

    # Generate a figure that shows naphthalene blood concentration vs. time.
    ymin = min(min(CVB_sim), min(c_data)) * 0.8
    ymax = max(max(CVB_sim), max(c_data)) * 1.1
    plot(t_sim, CVB_sim, type="l", lwd=2, lty=1, col=vcol[1],
         xlab="Time (min)", ylab="Concentration in Blood (ug/L)",
         ylim=c(ymin, ymax))
    points(t_data, c_data, pch=19, col=vcol[2], cex=1.0)
}

# Save optimal parameter values.
saveRDS(df_param, file="./Output/df_param_pde_indiv1.rds")

# Print cumulative computational time.
comp_time = difftime(Sys.time(), time0, units="mins")
cat("\nTime required to run this script:", comp_time[[1]], "minutes.\n")