#------------------------------------------------------------------------------
# skin_exp_plot_compare_all.R
#
# Author: Dustin Kapraun, US EPA, May 2019
#
# This script uses optimal pararameter results saved in the files
# "df_param_pde_indiv2.rds", "df_paramx_pde_all.rds", and
# "df_param_2c_indiv.rds" in the "Output" directory. These optimal sets of
# parameters for the models "naph_pbtk_pde" and "naph_pbtk_2c" for 10 subjects
# from the controlled JP8 skin exposure study of Kim et al. (2006) are used to
# plot model simulated blood concentrations along with observed blood
# concentrations for each of the 10 subjects. All plots are combined into a
# single 10-panel figure, which is saved to the "Output" directory as the file
# "skin_exp_fit_compare_all.tif".
#
# References:
# D Kim, ME Andersen, LA Nylander-French (2006), "A dermatotoxicokinetic model
# of human exposures to jet fuel," Toxicological Sciences 93(1), 22-23.
#------------------------------------------------------------------------------

# Set working directory to the directory containing this file.
script_dir = dirname(sys.frame(1)$ofile)
setwd(script_dir)

# Save some colors from the viridis palette. This vector of color-identification
# strings was generated by calling the function "viridis" from the package
# "viridis" with the argument "5"; i.e., "viridis(5)".
vcol = c("#440154FF", "#3B528BFF", "#21908CFF", "#5DC863FF", "#FDE725FF")

# Load data from Kim et al. (2006).
df_blood = read.csv("./Data/kim_blood_data.csv", row.names=1,
                    check.names=FALSE)
df_subject = read.csv("./Data/kim_subject_data.csv", row.names=1,
                      check.names=FALSE)

# Get number of subjects.
num_subjects = length(unique(df_blood[["Subject"]]))

# Get all unique subject identifiers.
subject_ids = unique(df_blood[["Subject"]])

# Load optimal parameter plot information for the two models for each subject.
df_plot1 = readRDS("./Output/df_plot_pde_indiv.rds")
df_plot2 = readRDS("./Output/df_plot_pde_all.rds")
df_plot3 = readRDS("./Output/df_plot_2c_indiv.rds")

# Construct the 10-panel figure.
file_name = "./Output/skin_exp_fit_compare_all.tif"
tiff(file_name, units="in", width=6.5, height=8, res=300)
par(mfrow=c(5, 2))   # 10 panels configured in 5 rows and 2 columns.
par(mar=c(1, 3, 0, 0), oma=c(4, 2, 1, 1))

# Generate a plot for each subject that includes 3 model fits and data.
for (sidx in seq(num_subjects)) {
    # Get relevant data (Kim et al., 2006) for this subject.
    subject_id = sidx

    # Blood concentration observation times (min).
    t_data = df_blood[df_blood["Subject"] == subject_id, "Time (min)"]

    # Observed blood concentations (ng/mL = ug/L).
    c_data = df_blood[df_blood["Subject"] == subject_id,
                      "Concentration (ng/mL)"]

    # Generate a figure that shows naphthalene blood concentration vs. time.
    # ymin = min(min(df_plot1[ , sidx+1]), min(c_data)) * 0.9
    # ymax = max(max(df_plot1[ , sidx+1]), max(c_data)) * 1.05
    if (sidx == 1) {
        ymin = 0.1
        ymax = 1.1
    } else if (sidx == 2) {
        ymin = 0.23
        ymax = 0.32
    } else if (sidx == 3) {
        ymin = 0.19
        ymax = 0.30
    } else if (sidx == 5) {
        ymin = 0.1
        ymax = 0.4
    } else {
        ymin = 0.14
        ymax = 0.32
    }
    plot(df_plot1[ , "t_sim"], df_plot1[ , sidx+1],
         type="l", lwd=2, lty=1, col=vcol[1],
         xlab=NA, ylab=NA,
         ylim=c(ymin, ymax),
         xaxt="none")
    lines(df_plot2[ , "t_sim"], df_plot2[ , sidx+1],
          lwd=2, lty=2, col=vcol[2])
    lines(df_plot3[ , "t_sim"], df_plot3[ , sidx+1],
          lwd=2, lty=3, col=vcol[3])
    points(t_data, c_data, pch=19, col=vcol[4], cex=1.0)
    title(paste("Subject", sidx), font.main=1, line=-1)

    # Include legend for first panel.
    if (sidx == 1) {
        legend("topright", c("PDE1", "PDE2", "2C", "Data"),
               col=vcol[1:4], lty=c(1, 2, 3, NA), lwd=c(2, 2, 2, NA),
               pch=c(NA, NA, NA, 19))
    }

    # Include x-axis labels for bottom panels.
    if (sidx %in% c(9, 10)) {
        axis(1, at= seq(0, 300, 50))
    }
}

# Add labels for the x- and y-axes.
mtext("Time (min)", side=1, outer=TRUE, cex=1.0, line=1.75)
mtext(expression(paste("Concentration in Blood (", mu, "g/L)")), side=2,
      outer=TRUE, cex=1.0, line=0.0)

# Close and finish the figure.
dev.off()