#------------------------------------------------------------------------------
# skin_exp_sim.R
#
# Author: Dustin Kapraun, US EPA, October 2019
#
# This script defines a function to perform a model simulation of skin exposure 
# based on the controlled JP8 skin exposure study of Kim et al. (2006). The
# function assumes that the script "RMCSim" has been sourced so that the
# function "run_model" is defined. It also assumes that the "load_model"
# function has been run with the appropriate model so that the functions
# "initParms" and "initStates" are appropriately defined.
#
# References:
# D Kim, ME Andersen, LA Nylander-French (2006), "A dermatotoxicokinetic model
# of human exposures to jet fuel," Toxicological Sciences 93(1), 22-23.
#------------------------------------------------------------------------------

# Define function that performs a simulation.
skin_exp_sim <- function(model_name, times, BWinit, exp_time, RIV, DSC, HSCJP8,
                         parms) {
    # Modify relevant parameters.
    parms["BWinit"] = BWinit    # Body mass of subject (g).
    parms["DSC"] = DSC          # SC diffusion parameter (cm^2/min).
    parms["HSCJP8"] = HSCJP8    # SC to JP-8 partition coefficient.
    parms = initParms(parms)

    # Get the molar mass of naphthalene (g/mol).
    MW1 = parms[["MW1"]]

    # Get default initial conditions.
    Y0 = initStates(parms)

    # Run a simulation assuming just the IV dosing to obtain steady state
    # conditions.
    TSTART = -1e5               # Start time (min).
    TSTOP = 0                   # Stop time (min).
    tvec = sort(c(TSTART, TSTOP, times[times < TSTOP]))
    Y0["RIV"] = RIV
    out0 = run_model(model_name, tvec, Y0=Y0, parms=parms)

    # Reset initial conditions using final values of state variables from the
    # previous simulation.
    Y0 = out0[nrow(out0), names(Y0)]

    # Run a simulation for the exposure period.
    TSTART = 0.0                # Start time (min).
    TSTOP = exp_time            # Stop time (min).
    tvec = sort(c(TSTART, TSTOP, times[(times > TSTART) & (times < TSTOP)]))
    parms["AEXP"] = 20          # Area of exposed skin (cm^2).
    parms["VWELL"] = 1.0        # Volume of JP-8 in exposure well (mL).
    parms = initParms(parms)
    VWELL = parms[["VWELL"]]
    Y0["AWELL"] = 0.003 * VWELL / MW1 * 1e9   # Amount of naphthalene in well
                                              # (nmol) assuming 3 mg per 1 mL
                                              # of JP-8.
    out1 = run_model(model_name, tvec, Y0=Y0, parms=parms)

    # Reset initial conditions using final values of state variables from the
    # previous simulation.
    Y0 = out1[nrow(out1), names(Y0)]

    # Run a simulation that encompasses all the observation times after the 
    # exposure period.
    TSTART = exp_time                   # Start time (min).
    TSTOP = max(c(exp_time + 1, times)) # Stop time (min).
    tvec = unique(sort(c(TSTART, TSTOP, times[times > TSTART])))
    parms["VWELL"] = 0          # Volume of skin exposure well (mL). Setting
                                # this to zero creates a zero flux condition at
                                # the outer surface of the stratum corneum.
    parms = initParms(parms)
    out2 = run_model(model_name, tvec, Y0=Y0, parms=parms)
           
    # Package results in a single data frame.
    out = rbind(out0[1:nrow(out0)-1, ], out1[1:nrow(out1)-1, ], out2)
    out = out[out[ , "time"] %in% times, ]
    
    # Return the data frame.
    return(out)
}